// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "CameraTexture.hpp"

using namespace Dawn;

CameraTexture::CameraTexture(SceneItem *item) :
  SceneItemComponent(item),
  camera(nullptr),
  renderTarget(32.0f, 32.0f),
  updateOrtho(true)
{
}

void CameraTexture::onStart() {
  if(this->camera == nullptr) this->camera = item->getComponent<Camera>();

  auto effectCamera = [&]{
    if(this->camera == nullptr) return teardown = [&]{};
    this->camera->renderTarget = &this->renderTarget;

    if(!this->updateOrtho) return teardown = [&]{};

    this->updateOrthoValues();
    return teardown = useEvent([&](float_t w, float_t h){
      this->updateOrthoValues();
    }, this->camera->eventRenderTargetResized);
  };
  
  useEffectWithTeardown(effectCamera, this->camera);
  useEffectWithTeardown(effectCamera, this->updateOrtho)();
}

void CameraTexture::updateOrthoValues() {
  assertNotNull(this->camera);
  float_t wr = this->camera->renderTarget->getWidth() / this->camera->renderTarget->getHeight();
  wr *= 0.5f;//TODO: allow this to be editable
  this->camera->orthoLeft = -wr;
  this->camera->orthoRight = wr;
}